package card

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/card"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strings"
	"time"

	"github.com/yl10/kit/guid"
)

// CardServ 系统处理
type CardServ struct {
	ctx     *utils.Context
	dao     *card.CardDAO
	custDao *customer.CustomerDAO
}

// NewCardServ 初始化
func NewCardServ(ctx *utils.Context) *CardServ {
	return &CardServ{
		ctx:     ctx,
		dao:     card.NewCardDAO(ctx),
		custDao: customer.NewCustomerDAO(ctx),
	}
}

// GetCardList 获取卡列表
func (s *CardServ) GetCardList(caseID string, pagenavi ...int) ([]model.TaCouponCard, int64, error) {
	filters := []string{}
	if caseID != "" {
		if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
			return nil, 0, err
		}

		filters = []string{
			"case_id='" + caseID + "'",
		}
	} else {
		casesRaw := s.ctx.Get("cases")
		if casesRaw == nil {
			return nil, 0, errors.New("请设置过滤案场")
		}

		cases := casesRaw.([]model.SysUserCase)
		caseIDs := []string{}
		for _, cs := range cases {
			caseIDs = append(caseIDs, cs.CaseId)
		}

		filters = []string{
			"case_id in ('" + strings.Join(caseIDs, "','") + "')",
		}
	}

	limit := utils.GetPageNaviLimit(pagenavi...)
	res, total, err := s.dao.GetCardList(filters, limit)
	if err != nil {
		utils.LogError("查询卡列表失败: " + err.Error())
		return nil, 0, errors.New("查询卡列表失败")
	}

	return res, total, nil
}

// EditCard 编辑卡 , 没有则新增
func (s *CardServ) EditCard(card *card.CardInfo) error {
	if card.CardId == "" {
		return s.SaveCard(card)
	} else {
		return s.UpdateCard(card)
	}
}

// SaveCard 保存卡
// 字段校验没有在此函数中设置
func (s *CardServ) SaveCard(cd *card.CardInfo) error {
	if cd.CaseId == "" {
		return errors.New("未指定案场")
	}

	cd.CardId = guid.NewGUIDString()

	// 保存主信息
	cardMain := cd.TaCouponCard
	if err := s.dao.AddCard(&cardMain); err != nil {
		utils.LogError("保存卡失败: " + err.Error())
		return errors.New("保存卡失败")
	}

	// 保存图片
	if err := s.dao.UpdateImages(cd.Images, cd.CardId); err != nil {
		utils.LogError("保存卡图片失败: " + err.Error())
		return errors.New("保存卡图片失败")
	}

	// 保存分享
	share := cd.Share
	share.CardId = cd.CardId
	if err := s.dao.SaveCardShare(share); err != nil {
		utils.LogError("保存卡分享失败: " + err.Error())
		return errors.New("保存卡分享失败")
	}

	// 保存关联内容
	if err := s.dao.UpdateTarget(cd.Targets, cd.CardId); err != nil {
		utils.LogError("保存卡关联内容失败: " + err.Error())
		return errors.New("保存卡关联内容失败")
	}

	return nil
}

// UpdateCard 更新卡
// 字段校验没有在此函数中设置
func (s *CardServ) UpdateCard(cd *card.CardInfo) error {
	if cd.CaseId == "" {
		return errors.New("未指定案场")
	}

	if cd.CardId == "" {
		return errors.New("未指定卡")
	}

	// 更新主信息
	cardMain := cd.TaCouponCard
	if err := s.dao.UpdateCard(&cardMain, []string{"Card_name"}); err != nil {
		utils.LogError("更新卡失败: " + err.Error())
		return errors.New("更新卡失败")
	}

	// 更新图片
	if err := s.dao.UpdateImages(cd.Images, cd.CardId); err != nil {
		utils.LogError("更新卡图片失败: " + err.Error())
		return errors.New("更新卡图片失败")
	}

	// 更新分享
	share := cd.Share
	share.CardId = cd.CardId
	cols := []string{"Card_share_info", "use_rule", "use_instruction"}
	if err := s.dao.UpdateCardShare(share, cols); err != nil {
		utils.LogError("更新卡分享失败: " + err.Error())
		return errors.New("更新卡分享失败")
	}

	// 关联内容 规则 不允许更新
	return nil
}

// GiveCard 赠送卡券
func (s *CardServ) GiveCard(CardID string, uids []string) error {
	// 当前用户
	userRaw := s.ctx.Get("user")
	if userRaw == nil {
		return errors.New("当前人员信息不正确, 请重试")
	}
	user := userRaw.(model.SysUser)

	card, err := s.dao.GetCardByID(CardID)
	if err != nil {
		utils.LogError("获取卡失败: " + err.Error())
		return errors.New("校验卡失败")
	}

	// 案场
	if err := utils.NewAuthEngine(s.ctx).CheckCase(card.CaseId); err != nil {
		return err
	}

	// 卡必须状态正常
	if card == nil || card.CardId == "" || card.Status != models.STATUS_NORMAL {
		return errors.New("卡已失效或者不存在")
	}

	for _, uid := range uids {
		// 获取客户信息
		cust, err := s.custDao.GetCustomerByID(uid)
		if err != nil {
			utils.LogError("查询客户信息失败: " + err.Error())
			return errors.New("校验客户信息失败")
		}

		if err := s.GiveCardTo(&user, cust, card); err != nil {
			return err
		}
	}

	return nil
}

// GiveCardTo 赠送卡
// 本方法会被多出调用, 方法内使用 Context 将是不安全的
func (s *CardServ) GiveCardTo(from *model.SysUser, to *model.TaCustomer, card *card.CardInfo) error {
	if from.UserId == "" {
		return errors.New("必须指定赠送人")
	}

	if to.CustomerId == "" {
		return errors.New("必须指定接收人")
	}

	// 卡必须状态正常
	if card.CardId == "" || card.Status != models.STATUS_NORMAL {
		return errors.New("卡已失效或者不存在")
	}

	// 计算卡券有效期
	startDate := card.StartDate
	endDate := card.EndDate

	// 检查用户是否已经有了该券
	custcardS, err := s.dao.CheckCustCard(to.CustomerId, card.CardId, startDate, endDate)
	if err != nil {
		utils.LogError("校验客户卡失败: " + err.Error())
		return errors.New("校验客户卡失败")
	}

	if custcardS != nil && len(custcardS) > 0 {
		return errors.New("客户 " + to.CustomerName + " 已经拥有该券")
	}

	// 放入我的卡券
	custcard := model.TaCustomerCard{
		CardId:           card.CardId,
		CustomerCardName: card.CardName,
		CustomerId:       to.CustomerId,
		StartDate:        startDate,
		EndDate:          endDate,
		ReceiveDate:      time.Now().Local(),
		CaseId:           card.CaseId,
		OrgId:            card.OrgId,
	}

	// 入库
	if err := s.dao.SaveCustomerCards([]model.TaCustomerCard{custcard}); err != nil {
		utils.LogError("保存客户卡失败: " + err.Error())
		return errors.New("保存客户卡失败")
	}

	// TODO
	// 券库存处理

	return nil
}

// GetCardByID 获取卡
// 未进行权限校验, 非安全的
func (s *CardServ) GetCardByID(cardID string) (*card.CardInfo, error) {
	card, err := s.dao.GetCardByID(cardID)
	if err != nil {
		utils.LogError("获取卡失败: " + err.Error())
		return nil, errors.New("校验卡失败")
	}

	return card, nil
}

// GetCardByIDWithCheck 获取卡
func (s *CardServ) GetCardByIDWithCheck(cardID string) (*card.CardInfo, error) {
	card, err := s.dao.GetCardByID(cardID)
	if err != nil {
		utils.LogError("获取卡失败: " + err.Error())
		return nil, errors.New("校验卡失败")
	}

	if err := utils.NewAuthEngine(s.ctx).CheckCase(card.CaseId); err != nil {
		return nil, err
	}

	return card, nil
}

// GetCardWithCustomer 获取优惠券明细
func (s *CardServ) GetCardWithCustomer(cardid string) (*card.CaseUsableCard, error) {
	if cardid == "" {
		return nil, errors.New("没有卡信息！")
	}
	cardDetail, err := s.dao.GetCardDetail(cardid)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return cardDetail, nil
}

// GetCardByCustomer 获取我的优惠券
func (s *CardServ) GetCardByCustomer() ([]model.TaCustomerCard, error) {
	org := s.ctx.Get("org").(model.SysOrg)
	customer := s.ctx.Get("customer").(model.TaCustomer)
	cards, err := s.dao.GetCardByCustomer(org.OrgId, customer.CustomerId)
	if err != nil {
		utils.LogError("获取体验卡失败: " + err.Error())
		return nil, errors.New("获取体验卡失败")
	}
	return cards, nil
}
