package coupon

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/coupon"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strings"
	"time"

	"github.com/astaxie/beego"

	"github.com/yl10/kit/guid"
)

// CouponServ 系统处理
type CouponServ struct {
	ctx     *utils.Context
	dao     *coupon.CouponDAO
	custDao *customer.CustomerDAO
}

// NewCouponServ 初始化
func NewCouponServ(ctx *utils.Context) *CouponServ {
	return &CouponServ{
		ctx:     ctx,
		dao:     coupon.NewCouponDAO(ctx),
		custDao: customer.NewCustomerDAO(ctx),
	}
}

// GetCouponList 获取优惠券列表
func (s *CouponServ) GetCouponList(caseID string, pagenavi ...int) ([]model.TaCoupon, int64, error) {
	filters := []string{}

	if caseID != "" {
		if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
			return nil, 0, err
		}

		filters = []string{
			"case_id='" + caseID + "'",
		}
	} else {
		casesRaw := s.ctx.Get("cases")
		if casesRaw == nil {
			return nil, 0, errors.New("请设置过滤案场")
		}

		cases := casesRaw.([]model.SysUserCase)
		caseIDs := []string{}
		for _, cs := range cases {
			caseIDs = append(caseIDs, cs.CaseId)
		}

		filters = []string{
			"case_id in ('" + strings.Join(caseIDs, "','") + "')",
		}
	}

	limit := utils.GetPageNaviLimit(pagenavi...)
	res, total, err := s.dao.GetCouponList(filters, limit)
	if err != nil {
		utils.LogError("查询优惠券列表失败: " + err.Error())
		return nil, 0, errors.New("查询优惠券列表失败")
	}

	return res, total, nil
}

// EditCoupon 编辑优惠券 , 没有则新增
func (s *CouponServ) EditCoupon(cp *coupon.CouponInfo) error {
	if cp.CouponId == "" {
		return s.SaveCoupon(cp)
	} else {
		return s.UpdateCoupon(cp)
	}
}

// SaveCoupon 保存优惠券
// 字段校验没有在此函数中设置
func (s *CouponServ) SaveCoupon(cp *coupon.CouponInfo) error {
	if cp.CaseId == "" {
		return errors.New("未指定案场")
	}

	cp.CouponId = guid.NewGUIDString()

	// 保存主信息
	cpMain := cp.TaCoupon
	if err := s.dao.SaveCoupon(&cpMain); err != nil {
		utils.LogError("保存优惠券失败: " + err.Error())
		return errors.New("保存优惠券失败")
	}

	// 保存图片
	if err := s.dao.UpdateImages(cp.Images, cp.CouponId); err != nil {
		utils.LogError("保存优惠券图片失败: " + err.Error())
		return errors.New("保存优惠券图片失败")
	}

	// 保存分享
	share := cp.Share
	share.CouponId = cp.CouponId
	// cols := []string{"coupon_share_info", "use_rule", "use_instruction"}
	if err := s.dao.SaveCouponShare(&share); err != nil {
		utils.LogError("保存优惠券分享失败: " + err.Error())
		return errors.New("保存优惠券分享失败")
	}

	// 保存关联内容
	if err := s.dao.UpdateTarget(cp.Targets, cp.CouponId); err != nil {
		utils.LogError("保存优惠券关联内容失败: " + err.Error())
		return errors.New("保存优惠券关联内容失败")
	}

	// 保存规则
	if err := s.dao.UpdateRule(cp.Rules, cp.CouponId); err != nil {
		utils.LogError("保存优惠券规则失败: " + err.Error())
		return errors.New("保存优惠券规则失败")
	}

	return nil
}

// UpdateCoupon 更新优惠券
// 字段校验没有在此函数中设置
func (s *CouponServ) UpdateCoupon(cp *coupon.CouponInfo) error {
	if cp.CaseId == "" {
		return errors.New("未指定案场")
	}

	if cp.CouponId == "" {
		return errors.New("未指定优惠券")
	}

	// 更新主信息
	cpMain := cp.TaCoupon
	if err := s.dao.UpdateCoupon(&cpMain, []string{"coupon_name",
		"coupon_type",
		"send_type",
		"video_url",
		"cover_url",
		"price",
		"is_all",
		"start_date",
		"end_date",
		"valid_days",
		"total_count",
		"case_id"}); err != nil {
		utils.LogError("更新优惠券失败: " + err.Error())
		return errors.New("更新优惠券失败")
	}

	// 更新图片
	if err := s.dao.UpdateImages(cp.Images, cp.CouponId); err != nil {
		utils.LogError("更新优惠券图片失败: " + err.Error())
		return errors.New("更新优惠券图片失败")
	}

	// 更新分享
	share := cp.Share
	share.CouponId = cp.CouponId
	cols := []string{"coupon_share_info", "use_rule", "use_instruction"}
	if err := s.dao.UpdateCouponShare(&share, cols); err != nil {
		utils.LogError("更新优惠券分享失败: " + err.Error())
		return errors.New("更新优惠券分享失败")
	}

	// 保存关联内容
	if err := s.dao.UpdateTarget(cp.Targets, cp.CouponId); err != nil {
		utils.LogError("保存优惠券关联内容失败: " + err.Error())
		return errors.New("保存优惠券关联内容失败")
	}

	// 保存规则
	if err := s.dao.UpdateRule(cp.Rules, cp.CouponId); err != nil {
		utils.LogError("保存优惠券规则失败: " + err.Error())
		return errors.New("保存优惠券规则失败")
	}

	return nil
}

// GiveCoupon 赠送卡券
func (s *CouponServ) GiveCoupon(couponID string, uids []string) error {
	// 当前用户
	userRaw := s.ctx.Get("user")
	if userRaw == nil {
		return errors.New("当前人员信息不正确, 请重试")
	}
	user := userRaw.(model.SysUser)

	cp, err := s.dao.GetCouponByID(couponID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return errors.New("校验优惠券失败")
	}

	// 案场
	if err := utils.NewAuthEngine(s.ctx).CheckCase(cp.CaseId); err != nil {
		return err
	}

	// 优惠券必须状态正常
	if cp == nil || cp.CouponId == "" || cp.Status != models.STATUS_NORMAL {
		return errors.New("优惠券已失效或者不存在")
	}

	now := time.Now().Local()
	if cp.ValidDays <= 0 {
		if cp.EndDate.Before(now) {
			return errors.New("优惠券已失效")
		}
	}

	for _, uid := range uids {
		// 获取客户信息
		cust, err := s.custDao.GetCustomerByID(uid)
		if err != nil {
			utils.LogError("查询客户信息失败: " + err.Error())
			return errors.New("校验客户信息失败")
		}

		if err := s.GiveCouponTo(&user, cust, cp); err != nil {
			return err
		}
	}

	return nil
}

// GiveCouponTo 赠送优惠券
// 本方法会被多出调用, 方法内使用 Context 将是不安全的
func (s *CouponServ) GiveCouponTo(from *model.SysUser, to *model.TaCustomer, cp *model.TaCoupon) error {
	if from.UserId == "" {
		return errors.New("必须指定赠送人")
	}

	if to.CustomerId == "" {
		return errors.New("必须指定接收人")
	}

	// 优惠券必须状态正常
	if cp.CouponId == "" || cp.Status != models.STATUS_NORMAL {
		return errors.New("优惠券已失效或者不存在")
	}

	now := time.Now().Local()
	if cp.ValidDays <= 0 {
		if cp.EndDate.Before(now) {
			return errors.New("优惠券已失效")
		}
	}

	if cp.TotalCount < cp.SentCount+1 {
		return errors.New("券已经没有库存！")
	}

	// 计算卡券有效期
	startDate := cp.StartDate
	endDate := cp.EndDate
	if cp.ValidDays > 0 {
		startDate = now
		hs := int64(cp.ValidDays * 24)
		endDate = now.Add(time.Hour * time.Duration(hs))
	}

	// 检查用户是否已经有了该券
	custCPS, err := s.dao.CheckCustCoupon(to.CustomerId, cp.CouponId, startDate, endDate)
	if err != nil {
		utils.LogError("校验客户优惠券失败: " + err.Error())
		return errors.New("校验客户优惠券失败")
	}
	beego.Error(custCPS)
	if custCPS != nil && len(custCPS) > 0 {
		return errors.New("客户 " + to.CustomerName + " 已经拥有该券")
	}

	// 放入我的卡券
	custCP := model.TaCustomerCoupon{
		CouponId:           cp.CouponId,
		CustomerCouponName: cp.CouponName,
		CustomerId:         to.CustomerId,
		StartDate:          startDate,
		EndDate:            endDate,
		ReceiveDate:        now,
		CaseId:             cp.CaseId,
		OrgId:              cp.OrgId,
	}

	// 放入赠送记录
	rec := model.TaCouponGiveRecord{
		GiftType:   models.GIVE_GIFT_COUPON,
		GiftId:     cp.CouponId,
		GiftName:   cp.CouponName,
		FromId:     from.UserId,
		FromName:   from.UserName,
		ToId:       to.CustomerId,
		ToName:     to.CustomerName,
		CreateDate: now,
	}

	// 入库
	if err := s.dao.SaveCustomerCoupons([]model.TaCustomerCoupon{custCP}); err != nil {
		utils.LogError("保存客户优惠券失败: " + err.Error())
		return errors.New("保存客户优惠券失败")
	}

	if err := s.dao.SaveCouponRecoreds([]model.TaCouponGiveRecord{rec}); err != nil {
		utils.LogError("保存优惠券赠送记录失败: " + err.Error())
		return errors.New("保存优惠券赠送记录失败")
	}

	// TODO
	// 券库存处理
	cp.SentCount = cp.SentCount + 1
	if err := s.dao.UpdateCoupon(cp, []string{"sent_count"}); err != nil {
		utils.LogError("更新优惠券库存失败: " + err.Error())
		return errors.New("更新优惠券库存失败")
	}
	return nil
}

// GetCouponByID 获取优惠券
// 未进行权限校验, 非安全的
func (s *CouponServ) GetCouponByID(cpID string) (*model.TaCoupon, error) {
	cp, err := s.dao.GetCouponByID(cpID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("校验优惠券失败")
	}

	return cp, nil
}

// GetCouponByIDWithCheck 获取优惠券
func (s *CouponServ) GetCouponByIDWithCheck(cpID string) (*coupon.CouponInfo, error) {
	cp, err := s.dao.GetCouponInfoByID(cpID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("校验优惠券失败")
	}

	if err := utils.NewAuthEngine(s.ctx).CheckCase(cp.CaseId); err != nil {
		return nil, err
	}

	return cp, nil
}

// GetCouponBySendType 根据发送类型获取优惠券
func (s *CouponServ) GetCouponBySendType(sendtype string) ([]model.TaCoupon, error) {
	cases := s.ctx.Get("cases").([]model.SysUserCase)
	caseIDs := ""
	for _, value := range cases {
		if caseIDs != "" {
			caseIDs = caseIDs + ","
		}
		caseIDs = caseIDs + value.CaseId
	}
	coupon, err := s.dao.GetCouponBySendType(caseIDs, sendtype)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return coupon, nil
}

// GetCouponWithCustomer 获取优惠券明细
func (s *CouponServ) GetCouponWithCustomer(couponid string) (*coupon.CaseCouponDetail, error) {
	if couponid == "" {
		return nil, errors.New("没有券信息！")
	}
	couponDetail, err := s.dao.GetCouponDetail(couponid)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return couponDetail, nil
}

// GetCouponByCustomer 获取我的优惠券
func (s *CouponServ) GetCouponByCustomer() ([]coupon.CustomerCoupon, error) {
	org := s.ctx.Get("org").(model.SysOrg)
	customer := s.ctx.Get("customer").(model.TaCustomer)
	coupons, err := s.dao.GetCouponByCustomer(org.OrgId, customer.CustomerId)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return coupons, nil
}
